#-- 
#  Copyright (C) 2008 Red Hat Inc.
#  
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#  
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#  
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# Author: Bryan Kearney <bkearney@redhat.com>
#--

require 'puppet/provider/package'

Puppet::Type.type(:mysql_root_user).provide(:mysql,
		:parent => Puppet::Provider) do

	desc "Use mysql as database."
	commands :mysql => '/usr/bin/mysql'
	commands :mysqladmin => '/usr/bin/mysqladmin'


	def root_password_set?
		set = false
		begin
			set = !((mysqladmin  "-u#{@resource[:name]}", "-p#{@resource[:password]}", "status")["Uptime"].nil?)
		rescue
			# If it failed, the password is probaly not set.
		end
		return set 
	end
	
	def query
		result = {}
		if root_password_set?
			result = {
						:name => @resource[:name],
						:password => @resource[:password],
						:ensure => :present
					}
		else 
			result = {
				:name => @resource[:name],
				:password => @resource[:password],
				:ensure => :absent
			}
		end 
		
		return result
	end

	def create
		mysqladmin  "-u#{@resource[:name]}", "password", @resource[:password]
	end

	def destroy
		#no-op
	end

	def exists?
		 root_password_set?
	end

end

