#-- 
#  Copyright (C) 2008 Red Hat Inc.
#  
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#  
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#  
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# Author: Bryan Kearney <bkearney@redhat.com>
#--

# Derived from David Schmitt's work
# http://git.black.co.at/?p=module-mysql

require 'puppet/provider/package'

Puppet::Type.type(:mysql_user).provide(:mysql,
		# T'is funny business, this code is quite generic
		:parent => Puppet::Provider) do

	desc "Use mysql as database."
	commands :mysql => '/usr/bin/mysql'
	commands :mysqladmin => '/usr/bin/mysqladmin'

	# retrieve the current set of mysql users
#	def self.instances
#		users = Array.new()
#
	#	cmd = "#{command(:mysql)} mysql -u#{@resource[:root_user]} -p#{@resource[:root_user_password]} -NBe 'select concat(user, \"@\", host), password from user'"
	#	execpipe(cmd) do |process|
	#		process.each do |line|
	#			users << new( query_line_to_hash(line) )
	#		end
	#	end
	#	return users
	#end

	def self.query_line_to_hash(line)
		fields = line.chomp.split(/\t/)
		{
			:name => fields[0],
			:password_hash => fields[1],
			:ensure => :present
		}
	end

	def mysql_flush 
		mysqladmin  "-u#{@resource[:root_user]}", "-p#{@resource[:root_user_password]}", "flush-privileges"
	end

	def query
		result = {}

		cmd = "#{command(:mysql)} mysql -u#{@resource[:root_user]} -p#{@resource[:root_user_password]} -NBe 'select concat(user, \"@\", host), password from user where concat(user, \"@\", host) = \"%s\"'" % @resource[:name]
		execpipe(cmd) do |process|
			process.each do |line|
				unless result.empty?
					raise Puppet::Error,
						"Got multiple results for user '%s'" % @resource[:name]
				end
				result = query_line_to_hash(line)
			end
		end
		result
	end

	def create
		mysql "-u#{@resource[:root_user]}", "-p#{@resource[:root_user_password]}" ,"-e", "create user '%s' identified by PASSWORD '%s'" % [ @resource[:name].sub("@", "'@'"), @resource.should(:password_hash) ]
		mysql_flush
	end

	def destroy
		mysql "-u#{@resource[:root_user]}", "-p#{@resource[:root_user_password]}",  "-e", "drop user '%s'" % @resource[:name].sub("@", "'@'")
		mysql_flush
	end

	def exists?
		not (mysql"-u#{@resource[:root_user]}", "-p#{@resource[:root_user_password]}",  "-NBe", "select '1' from mysql.user where CONCAT(user, '@', host) = '%s'" % @resource[:name]).empty?
	end

	def password_hash
		@property_hash[:password_hash]
	end

	def password_hash=(string)
		mysql "-u#{@resource[:root_user]}", "-p#{@resource[:root_user_password]}",  "-e", "SET PASSWORD FOR '%s' = '%s'" % [ @resource[:name].sub("@", "'@'"), string ]
		mysql_flush
	end
end

