#-- 
#  Copyright (C) 2008 Red Hat Inc.
#  
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#  
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#  
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# Author: Bryan Kearney <bkearney@redhat.com>
#--

# Derived from David Schmitt's work
# http://git.black.co.at/?p=module-mysql

Puppet::Type.newtype(:mysql_grant) do
	@doc = "Manage a database user's rights."
	#ensurable

	autorequire :mysql_db do
		# puts "Starting db autoreq for %s" % self[:name]
		reqs = []
		matches = self[:name].match(/^([^@]+)@([^\/]+)\/(.+)$/)
		unless matches.nil?
			reqs << matches[3]
		end
		# puts "Autoreq: '%s'" % reqs.join(" ")
		reqs
	end

	autorequire :mysql_user do
		# puts "Starting user autoreq for %s" % self[:name]
		reqs = []
		matches = self[:name].match(/^([^@]+)@([^\/]+).*$/)
		unless matches.nil?
			reqs << "%s@%s" % [ matches[1], matches[2] ]
		end
		# puts "Autoreq: '%s'" % reqs.join(" ")
		reqs
	end

	newparam(:name) do
		desc "The primary key: either user@host for global privilges or user@host/database for database specific privileges"
	end
	
	newparam(:root_user) do
			desc "root user"
	end
	newparam(:root_user_password) do
			desc "root user password"
	end	
		
	newproperty(:privileges, :array_matching => :all) do
		desc "The privileges the user should have. The possible values are implementation dependent."
		munge do |v|
			symbolize(v)
		end

		def should_to_s(newvalue = @should)
			if newvalue
				unless newvalue.is_a?(Array)
					newvalue = [ newvalue ]
				end
				newvalue.collect do |v| v.to_s end.sort.join ", "
			else
				nil
			end
		end

		def is_to_s(currentvalue = @is)
			if currentvalue
				unless currentvalue.is_a?(Array)
					currentvalue = [ currentvalue ]
				end
				currentvalue.collect do |v| v.to_s end.sort.join ", "
			else
				nil
			end
		end

		# use the sorted outputs for comparison
		def insync?(is)
			if defined? @should and @should
				case self.should_to_s 
				when "all"
					self.provider.all_privs_set?
				when self.is_to_s(is)
					true
				else
					false
				end
			else
				true
			end
		end

	end
end

